"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Redactor = void 0;
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const stream = require("stream");
class Redactor extends stream.Transform {
    constructor(redacted, options = {}) {
        super(options);
        this.redacted = redacted;
    }
    _transform(chunk, encoding, callback) {
        const stringified = chunk.toString();
        const redacted = this.redacted.reduce((acc, elem) => {
            if (!elem) {
                return acc;
            }
            return acc.replace(elem, '<REDACTED>');
        }, stringified);
        this.push(redacted);
        callback();
    }
}
exports.Redactor = Redactor;
exports.default = Redactor;
